clc
close all;
clear all;

%% Define buffers and global variable
pBuffer0 = libpointer('int16Ptr',ones(1024,1)); %Buffer pointer defined for AWG waveform                                       
pBuffer = libpointer('int16Ptr',zeros(1024,1)); %Buffer for getting valuse from the scope

global data;
data.unithandle = 0;
data.serial = [];
data.channelcount = 2; % Number of channels on the device
data.ps2000aInputRanges = [10, 20, 50, 100, 200, 500, 1000, 2000, 5000, ...
    10000, 20000, 50000]; % HM - Voltage ranges (in mV)

PS2000A_MIN_DWELL_COUNT = 10;
TRUE = 1;
FALSE = 0;

%% load_library
loadlibrary('PS2000a.dll','PS2000aMFile');
if ~libisloaded('PS2000a')
    error('library PS2000a.dll or PS2000aMFile not found') %check if the library is loaded
end

%% Load enumerations and structures
[methodinfo, structs, enuminfo, Thunklibname] = PS2000aMFile;
%% open unit
[status_open, unithandle, serial] = calllib('PS2000a','ps2000aOpenUnit', data.unithandle, data.serial)
data.unithandle = unithandle;

%% set_SCOPE
%% setting scope channel
channel_a = 'PS2000A_CHANNEL_A'; %'PS2000A_CHANNEL A';% use channel A of scope 
cha_enabled = TRUE ; % when 'TRUE', enable
type = 'PS2000A_AC'; % AC coupled mode
range = 'PS2000A_2V'; % input voltage level

% Obtain the voltage range used as a numerical value (use +1 for index offset)
voltage_range = data.ps2000aInputRanges(enuminfo.enPS2000ARange.PS2000A_2V + 1); 
analogOffset = 0; % no_offset

% Enable channel A
status_set_scope_cha = calllib('PS2000a','ps2000aSetChannel', ...
    data.unithandle,channel_a,cha_enabled,type,range,analogOffset)

%Disable channel B
channel_b = 'PS2000A_CHANNEL_B'; %'PS2000A_CHANNEL A';% use channel A of scope 
chb_enabled = FALSE; % when 'TRUE', enable

status_set_scope_chb = calllib('PS2000a','ps2000aSetChannel', ...
    data.unithandle, channel_b, chb_enabled, type, range, analogOffset)

%% Max ADC value

% Use this to help convert from ADC to voltage values
max_value = 0; % MATLAB automatically converts this to a pointer

[status_max_value, max_value] = calllib('PS2000a','ps2000aMaximumValue', data.unithandle, max_value)
%% set_AWG 
% setting pulse generator parameters and loading the arbitrary waveform
frequency = 1; % HM - desired frequency in Hertz

%offsetVoltage = 1000000; %1V offset
offsetVoltage = 1000000; %1V offset
pkToPk = 1000000; %1V peak to peak

%HM - This gives the voltage range of the signal
awg_voltage_range = pkToPk/(2 * 1000);

% Test to create a sine wave
%x = [0:(2*pi/1023):(2*pi)];

% Populate buffer containing waveform in millivolt values
arbitraryWaveformMV = awg_voltage_range * get(pBuffer0,'Value');
%arbitraryWaveformMV = awg_voltage_range * sin(x');

arbitraryWaveformSize = length(arbitraryWaveformMV); %buffer size
% HM - moved these here to use arbitraryWaveformSize variable

arbitraryWaveform = mv2adc(arbitraryWaveformMV, awg_voltage_range, max_value); % Convert values to ADC counts


% HM - updated to include frequency
startDeltaPhase = frequency * arbitraryWaveformSize * 2^(32-14) / 20000000; % this value is to get 1Hz of frequency, from the programmer's guide pp73
stopDeltaPhase = frequency * arbitraryWaveformSize * 2^(32-14) / 20000000;

deltaPhaseIncrement = 0;
dwellCount = PS2000A_MIN_DWELL_COUNT;

sweepType = 'PS2000A_UP';
operation = 'PS2000A_ES_OFF';
indexMode = 'PS2000A_SINGLE';
shots = 1; % number of cycles to be produced, 1cycle
sweeps = 0; % no_sweep of frequency
triggerType ='PS2000A_SIGGEN_RISING';
%triggerSource ='PS2000A_SIGGEN_SCOPE_TRIG';
triggerSource ='PS2000A_SIGGEN_SOFT_TRIG';
%triggerSource ='PS2000A_SIGGEN_NONE';
extInThreshold = 0; % no_external triger
status_set_ARB_GEN = calllib('PS2000a', 'ps2000aSetSigGenArbitrary',data.unithandle, offsetVoltage,pkToPk,startDeltaPhase,stopDeltaPhase,deltaPhaseIncrement,dwellCount,arbitraryWaveform,arbitraryWaveformSize,sweepType,operation,indexMode,shots,sweeps,triggerType,triggerSource,extInThreshold) 

%% getting time_base
timebase = 1252; % 10us
noSamples = length(get(pBuffer0,'Value')); % number of samples required
timeInterval = 0; % variable to be returned
oversample = 0; %no_oversample
maxSamples = 0; % variable to be returned
[status_get_time_base,timeInterval,maxSamples] = calllib('PS2000a','ps2000aGetTimebase',data.unithandle,timebase, noSamples, timeInterval, oversample, maxSamples, 0)
%get(timeInterval,'Value')
%get(maxSamples,'Value')


%% setting trigger 
enable = TRUE ; %non-zero value to set the trigger
source = channel_a; %the channel on which to trigger.

threshold_mV = 1000; %the threshold value in milliVolts at which the trigger will fire
threshold_adc = mv2adc(threshold_mV , voltage_range, max_value); %the ADC count at which the trigger will fire
direction = 'PS2000A_ABOVE';
delay = 0;
autoTrigger_ms = 1000;
status_setsTrg = calllib('PS2000a','ps2000aSetSimpleTrigger',data.unithandle,enable, source, threshold_adc, direction, delay, autoTrigger_ms)

%% set_buffer

% Set buffer prior to run block

channel = 0 %Have to define as integer value NOT enum
buffer = pBuffer;
bufferLth = length(get(buffer,'Value'));
segmentIndex = 0;
mode = 'PS2000A_RATIO_MODE_NONE';
status_setbf = calllib('PS2000a','ps2000aSetDataBuffer',data.unithandle, channel, buffer, bufferLth, segmentIndex, mode)

%% Trigger the Signal Generator

state = 0; %'PS2000A_SIGGEN_RISING'
status_sig_gen_sw_ctrl = calllib('PS2000a','ps2000aSigGenSoftwareControl', data.unithandle, state)

%% run block
noOfPreTriggerSamples = 0;
noOfPostTriggerSamples = length(get(pBuffer0,'Value'));
%timebase = 1; HM - no need to define the timebase value again
oversample = 0;
timeIndisposedMs = [];
segmentIndex = 0;
lpReady = [];
pParameter = [];

status_setRB = calllib('PS2000a','ps2000aRunBlock',data.unithandle, noOfPreTriggerSamples, noOfPostTriggerSamples, timebase, oversample,timeIndisposedMs, segmentIndex, lpReady, pParameter)

%% is ready
ready = 0; % No need to define as pointer as MATLAB automatically converts the data type
%ready = libpointer('int16Ptr',zeros(1,1));%pointer indicating readiness to transfer data from scope to pc

% Poll the driver until it indicates that data is ready for collection

while ready == 0
    
    [status_isrdy, ready] = calllib('PS2000a','ps2000aIsReady',data.unithandle,ready)
    pause(0.25) %Insert a pause to give the driver some time
end

%get(ready,'Value')


%% get_Value
startIndex = 0;
noOfSamples = length(get(pBuffer,'Value'));
downSampleRatio = 1;
downSampleRatioMode = 'PS2000A_RATIO_MODE_NONE';
segmentIndex = 0;
overflow = 0;
status_get_value = calllib('PS2000a','ps2000aGetValues',data.unithandle,startIndex, noOfSamples, 1, downSampleRatioMode, segmentIndex, overflow)
%% Stop data collection

% HM- Need to call the stop function to tell the unit to stop collecting data
status_stop = calllib('PS2000a', 'ps2000aStop', data.unithandle)

%% Plot the data as milliVolt values - HM

figure;
ch_a_values = get(buffer, 'Value');

%ch_a_values_mv = zeros(length(ch_a_values), 1);

ch_a_values_mv = adc2mv(ch_a_values, voltage_range, max_value); 
    

% Array of time values
t_ns = ([0:(length(ch_a_values) - 1)] * double(timeInterval));

% Plot mV
plot(t_ns', ch_a_values_mv)
title('Voltage versus Time')
xlabel('Time (ns)')
ylabel('Voltage (mV)');

% Plot ADC Counts
figure;
plot(t_ns', ch_a_values);
title('ADC Count versus Time')
xlabel('Time (ns)')
ylabel('ADC Count');

%% Close the unit
status_close = calllib('PS2000a', 'ps2000aCloseUnit', data.unithandle)
unloadlibrary('PS2000a');